package org.robokind.headless.speech;

import java.util.Properties;
import java.util.logging.Logger;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.robokind.api.common.lifecycle.utils.SimpleLifecycle;
import org.robokind.api.common.osgi.lifecycle.OSGiComponent;
import org.jflux.api.core.Adapter;
import org.robokind.api.messaging.services.ServiceCommand;
import org.robokind.api.messaging.services.ServiceError;
import org.robokind.api.speech.SpeechConfig;
import org.robokind.api.speech.SpeechEventList;
import org.robokind.api.speech.SpeechRequest;
import org.robokind.api.speech.SpeechRequestFactory;
import org.robokind.api.speech.SpeechService;
import org.robokind.api.speech.lifecycle.RemoteSpeechServiceLifecycle;
import org.robokind.api.speech.viseme.lifecycle.VisemeEventNotifierLifecycle;
import org.robokind.avrogen.messaging.ServiceCommandRecord;
import org.robokind.avrogen.messaging.ServiceErrorRecord;
import org.robokind.avrogen.speech.SpeechConfigRecord;
import org.robokind.avrogen.speech.SpeechEventListRecord;
import org.robokind.avrogen.speech.SpeechRequestRecord;
import org.robokind.impl.messaging.lifecycle.JMSAvroAsyncReceiverLifecycle;
import org.robokind.impl.messaging.lifecycle.JMSAvroMessageSenderLifecycle;
import org.robokind.impl.messaging.services.PortableServiceCommand;
import org.robokind.impl.messaging.services.PortableServiceError;
import org.robokind.impl.speech.PortableSpeechConfig;
import org.robokind.impl.speech.PortableSpeechEventList;
import org.robokind.impl.speech.PortableSpeechRequest;

public class Activator implements BundleActivator {
    private final static Logger theLogger = 
            Logger.getLogger(Activator.class.getName());
    
    private final static String SPEECH_SERVICE_ID = "speechService";
    private final static String CONNECTION_ID = "speechConnection";
    private final static String COMMAND_DEST_ID = "speechCommand";
    private final static String CONFIG_DEST_ID = "speechConfig";
    private final static String ERROR_DEST_ID = "speechError";
    private final static String REQUEST_DEST_ID = "speechRequest";
    private final static String EVENT_DEST_ID = "speechEvent";
    private final static String COMMAND_SENDER_ID = "speechCommand";
    private final static String CONFIG_SENDER_ID = "speechConfig";
    private final static String ERROR_RECEIVER_ID = "speechError";
    private final static String REQUEST_SENDER_ID = "speechRequest";
    private final static String EVENT_RECEIVER_ID = "speechEvent";
    

    @Override
    public void start(BundleContext context) throws Exception {
        theLogger.info("Launching JMS SpeechService Provider.");
        launchVisemeNotifier(context, SPEECH_SERVICE_ID);
        launchRemoteClient(context, 
                SPEECH_SERVICE_ID, SPEECH_SERVICE_ID, CONNECTION_ID, 
                COMMAND_DEST_ID, COMMAND_SENDER_ID, 
                CONFIG_DEST_ID, CONFIG_SENDER_ID, 
                ERROR_DEST_ID, ERROR_RECEIVER_ID, 
                REQUEST_DEST_ID, REQUEST_SENDER_ID, 
                EVENT_DEST_ID, EVENT_RECEIVER_ID);
        new OSGiComponent(context, 
                new SimpleLifecycle(new PortableSpeechRequest.Factory(), 
                        SpeechRequestFactory.class)).start();
    }
    
    private void launchVisemeNotifier(
            BundleContext context, String speechServiceId){
        theLogger.info("Launching Dynamic VisemeNotifier Service.");
        new OSGiComponent(context,
                new VisemeEventNotifierLifecycle(speechServiceId)
        ).start();
        theLogger.info("Dynamic VisemeNotifier Service Launched.");
    }
    
    private void launchRemoteClient(BundleContext context, 
            String speechClientId, String speechHostId, String connectionId, 
            String commandDestId, String commandSenderId,
            String configDestId, String configSenderId, 
            String errorDestId, String errorReceiverId,
            String speechReqDestId, String speechRequestSenderId,
            String speechEventsDestId, String speechEventsReceiverId){
        Properties props = new Properties();
        props.put(SpeechService.PROP_ID, speechClientId);
        
        startClientMessengers(context, props, connectionId,
                commandDestId, commandSenderId, 
                configDestId, configSenderId, 
                new PortableSpeechConfig.MessageRecordAdapter(), 
                SpeechConfig.class, SpeechConfigRecord.class, 
                errorDestId, errorReceiverId);
        startSpeechMessengers(context, props, connectionId, 
                speechReqDestId, speechRequestSenderId, 
                speechEventsDestId, speechEventsReceiverId);
        launchRemoteSpeechClient(context, speechClientId, speechHostId, 
                commandSenderId, configSenderId, errorReceiverId, 
                speechRequestSenderId, speechEventsReceiverId);
    }
    
    private void launchRemoteSpeechClient(BundleContext context, 
            String speechClientId, String speechHostId,
            String commandSenderId, String configSenderId, 
            String errorReceiverId, String speechRequestSenderId,
            String speechEventsReceiverId){
        RemoteSpeechServiceLifecycle lifecycle =
                new RemoteSpeechServiceLifecycle(speechClientId, speechHostId, 
                        commandSenderId, configSenderId, errorReceiverId, 
                        speechRequestSenderId, speechEventsReceiverId);
        OSGiComponent speechComp = new OSGiComponent(context, lifecycle);
        speechComp.start();
    }
    
    private static <Msg,Rec> void startClientMessengers(BundleContext context, 
            Properties groupProps, String connectionId, 
            String commandDestId, String commandSenderId,
            String configDestId, String configSenderId, 
            Adapter<Msg,Rec> configMsgRecAdapter, 
            Class<Msg> msgClass, Class<Rec> recClass,
            String errorDestId, String errorReceiverId){
        JMSAvroMessageSenderLifecycle commandSender = 
                new JMSAvroMessageSenderLifecycle(
                        new PortableServiceCommand.MessageRecordAdapter(), 
                        ServiceCommand.class, ServiceCommandRecord.class, 
                        commandSenderId, connectionId, commandDestId);
        new OSGiComponent(context, commandSender, groupProps).start();
        
        JMSAvroMessageSenderLifecycle configSender = 
                new JMSAvroMessageSenderLifecycle(
                        configMsgRecAdapter, msgClass, recClass, 
                        configSenderId, connectionId, configDestId);
        new OSGiComponent(context, configSender, groupProps).start();
        
        JMSAvroAsyncReceiverLifecycle errorReceiver = 
                new JMSAvroAsyncReceiverLifecycle(
                        new PortableServiceError.RecordMessageAdapter(), 
                        ServiceError.class, ServiceErrorRecord.class, 
                        ServiceErrorRecord.SCHEMA$, errorReceiverId, 
                        connectionId, errorDestId);
        new OSGiComponent(context, errorReceiver, groupProps).start();
    }
    
    private static void startSpeechMessengers(BundleContext context, 
            Properties groupProps, String connectionId, 
            String speechReqDestId, String speechRequestSenderId,
            String speechEventsDestId, String speechEventsReceiverId){
        
        JMSAvroMessageSenderLifecycle speechRequestSender = 
                new JMSAvroMessageSenderLifecycle(
                        new PortableSpeechRequest.MessageRecordAdapter(), 
                        SpeechRequest.class, SpeechRequestRecord.class, 
                        speechRequestSenderId, connectionId, speechReqDestId);
        new OSGiComponent(context, speechRequestSender, groupProps).start();
        
        JMSAvroAsyncReceiverLifecycle speechEventsReceiver = 
                new JMSAvroAsyncReceiverLifecycle(
                        new PortableSpeechEventList.RecordMessageAdapter(), 
                        SpeechEventList.class, SpeechEventListRecord.class, 
                        SpeechEventListRecord.SCHEMA$, speechEventsReceiverId, 
                        connectionId, speechEventsDestId);
        new OSGiComponent(context, speechEventsReceiver, groupProps).start();
    }

    @Override
    public void stop(BundleContext context) throws Exception {
        // TODO add deactivation code here
    }
}
